// https://github.com/vuejs/vue-cli/blob/dev/packages/%40vue/cli-ui/src/main.js#L12
const gql = window.gql

export default {
  beforeCreate () {
    const DARK_MODE = gql`
      query darkMode {
        darkMode @client
      }
    `
    const CONNECTED = gql`
      query connected {
        connected @client
      }
    `
    if (this.$options.clientState) {
      const newData = {
        connected: CONNECTED,
        darkMode: DARK_MODE
      }
      this.$options.apollo = {
        ...this.$options.apollo,
        ...newData
      }
      // Proxy prop on `this`
      for (const key in newData) {
        Object.defineProperty(this, key, {
          get: () => this.$data.$apolloData.data[key],
          enumerable: true,
          configurable: true
        })
      }
    }
  },
  methods: {
    async $get (id) {
      const SHARED_DATA = gql`
        fragment sharedData on SharedData {
          id
          value
        }
        query sharedData ($id: ID!, $projectId: ID!) {
          sharedData (id: $id, projectId: $projectId) {
            ...sharedData
          }
        }
      `
      const projectId = this.$getProjectId()
      const result = await this.$apollo.query({
        query: SHARED_DATA,
        variables: {
          id: id,
          projectId
        }
      })
      // it's a bug of vue cli
      // return result.sharedData.value
      // https://github.com/vuejs/vue-cli/blob/dev/packages/%40vue/cli-ui/src/util/shared-data.js#L79
      return result.data.sharedData.value
    },
    async $call (id, params) {
      const result = await this.$callPluginAction(id, params)
      return result ? result.data.pluginActionCall.results[0] : result
    }
  }
}
