// https://github.com/vuejs/vue-cli/tree/dev/packages/%40vue/cli-shared-utils
import chalk from 'chalk'
import padStart from 'string.prototype.padstart'

const format = (label, msg) => {
  return msg.split('\n').map((line, i) => {
    return i === 0
      ? `${label} ${line}`
      : padStart(line, $chalk.reset(label).length)
  }).join('\n')
}

const $chalk = chalk.constructor({ level: 3 })

const chalkTag = msg => $chalk.bgBlackBright.white.dim(` ${msg} `)

const chalkMyTag = ({ text, bg, color }) => {
  return $chalk[bg][color].bold(text)
}

export default class Logger {
  constructor (terminal) {
    this._terminal = terminal
  }

  async _log (message) {
    this._terminal.addLog({
      text: message,
      type: 'stdout'
    })
  }

  write (msg) {
    this._log(msg)
  }

  newline () {
    this._log('\n')
  }

  log (msg = '', tag = null) {
    const _msg = tag ? format(chalkMyTag(tag), msg) : msg
    this._log(_msg)
  }

  info (msg, tag = null) {
    const _msg = format($chalk.bgBlue.black(' INFO ') + (tag ? chalkTag(tag) : ''), msg)
    this._log(_msg)
  }

  done (msg, tag = null) {
    const _msg = format($chalk.bgGreen.black(' DONE ') + (tag ? chalkTag(tag) : ''), msg)
    this._log(_msg)
  }

  warn (msg, tag = null) {
    const _msg = format($chalk.bgYellow.black(' WARN ') + (tag ? chalkTag(tag) : ''), $chalk.yellow(msg))
    this._log(_msg)
  }

  error (msg, tag = null) {
    const _msg = format($chalk.bgRed(' EROR ') + (tag ? chalkTag(tag) : ''), $chalk.red(msg))
    this._log(_msg)
    if (msg instanceof Error) {
      this._log(msg.stack)
    }
  }
}
